@@ -39,7 +39,8 @@ betabin <- function(formula, random, data = NULL, link = c("logit", "cloglog"),
   gf <- formula(paste(formula[2], "~", gf3))
 
 # get the data
-  if(missing(data)) data <- environment(gf)
+  if(missing(data))
+    data <- environment(gf)
   
 # model frame and model matrix for the fixed effects
   mb <- match(c("formula", "data", "na.action"), names(mf), 0)
@@ -49,21 +50,19 @@ betabin <- function(formula, random, data = NULL, link = c("logit", "cloglog"),
   names(mfb)[2] <- "formula"
   mfb <- eval(mfb, parent.frame())
   mt <- attr(mfb, "terms")
+  modmatrix.b <- if(!is.empty.model(mt)) model.matrix(mt, mfb) else matrix(, NROW(Y), 0)
   Y <- model.response(mfb, "numeric")
-  modmatrix.b <- if(!is.empty.model(mt)) model.matrix(mt, mfb, contrasts) else matrix(, NROW(Y), 0)
   weights <- model.weights(mfb)
   if(!is.null(weights) && any(weights < 0))
     stop("Negative wts not allowed")
 
-### change on 12th July 2005 (check lines with weight = 0)
+## Check lines with weight = 0
   n <- rowSums(Y)
   y <- Y[, 1]
   
   if(any(n == 0))
     warning("The data set contains at least one line with weight = 0.\n")
 
-### end change
-
 # model frame and model matrix for the correlation structure
   mr <- match(c("random", "data", "na.action"), names(mf), 0)
   mr <- mf[c(1, mr)]
@@ -107,6 +106,7 @@ betabin <- function(formula, random, data = NULL, link = c("logit", "cloglog"),
 
   if(!is.null(unlist(fixpar))) 
     param.ini[fixpar[[1]]] <- fixpar[[2]]  
+
   # minuslogL
   minuslogL <- function(param){
    if(!is.null(unlist(fixpar)))
@@ -116,19 +116,11 @@ betabin <- function(formula, random, data = NULL, link = c("logit", "cloglog"),
    p <- invlink(eta, type = link)
    phi <- as.vector(modmatrix.phi %*% param[(nb.b + 1):(nb.b + nb.phi)])
 
-# old code changed on 4th May 2006
-#   fn <- ifelse(phi == 0,
-#           dbinom(x = y, size = n, prob = p, log = TRUE),
-#           lchoose(n,y) + lbeta(p * (1-phi)/phi + y, (1-p) * (1-phi)/phi + n-y) - lbeta(p*(1-phi)/phi, (1-p)*(1-phi)/phi))
-#    fn <- sum(fn)
-
-# new code 4th May 2006
    cnd <- phi == 0
    f1 <- dbinom(x = y[cnd], size = n[cnd], prob = p[cnd], log = TRUE) 
    n2 <- n[!cnd] ; y2 <- y[!cnd] ; p2 <- p[!cnd] ; phi2 <- phi[!cnd]
    f2 <- lchoose(n2, y2) + lbeta(p2 * (1 - phi2)/phi2 + y2, (1 - p2) * (1 - phi2)/phi2 + n2 - y2) - lbeta(p2 * (1 - phi2)/phi2, (1 - p2) * (1 - phi2)/phi2)
    fn <- sum(c(f1, f2))       
-# end new code 4th May 2006
 
     if(!is.finite(fn))
       fn <- -1e20
@@ -165,24 +157,37 @@ betabin <- function(formula, random, data = NULL, link = c("logit", "cloglog"),
 
   if(!is.null(unlist(fixpar)))
     param[fixpar[[1]]] <- fixpar[[2]]
-  varparam <- NA
+
+###  Beginning changes provided by Matthieu Lesnoff on Jan 8th 2007
+  H <- H.singular <- NA
+  varparam <- matrix(NA)
   if(hessian){
     H <- res$hessian
-    if(is.null(unlist(fixpar)))
-      varparam <- qr.solve(H)
+    if(is.null(unlist(fixpar))){
+      H.singular <- if(qr(H)$rank < nrow(H)) TRUE else FALSE
+      if(!H.singular)
+        varparam <- qr.solve(H)
+      else
+        warning("The hessian matrix was singular.\n")
+      }
     else{
       idparam <- 1:(nb.b + nb.phi)
       idestim <- idparam[-fixpar[[1]]]
       Hr <- H[-fixpar[[1]], -fixpar[[1]]]
-      Vr <- solve(Hr)
-      dimnames(Vr) <- list(idestim, idestim)
-#      varparam <- H
-#      varparam[idestim, idestim] <- Vr
-      varparam <- matrix(rep(NA, NCOL(H) * NROW(H)), ncol = NCOL(H))
-      varparam[idestim, idestim] <- Vr
+      H.singular <- if(qr(Hr)$rank < nrow(Hr)) TRUE else FALSE
+      if(!H.singular) {
+        Vr <- solve(Hr); dimnames(Vr) <- list(idestim, idestim)
+        varparam <- matrix(rep(NA, NROW(H) * NCOL(H)), ncol = NCOL(H))
+        varparam[idestim, idestim] <- Vr
+        }
       }
     }
-  if(!is.null(dim(varparam)))
+  else
+    varparam <- matrix(NA)
+
+### End of changes
+
+  if(any(!is.na(varparam)))
     dimnames(varparam) <- list(nam, nam)
 
   nbpar <- if(is.null(unlist(fixpar)))
@@ -196,10 +201,15 @@ betabin <- function(formula, random, data = NULL, link = c("logit", "cloglog"),
   df.residual <- sum(n > 0) - nbpar
   iterations <- res$counts[1]
   code <- res$convergence
+  msg <- if(!is.null(res$message)) res$message else character(0)
+
+  if(code != 0)
+    warning("\Possible convergence problem. Optimization process code: ", code, " (see ?optim).\n")
 
 # Output
   new(Class = "glimML", CALL = CALL, link = link, method = "BB", data = data, formula = formula, random = random, 
-      param = param, varparam = varparam,  logL = logL, logL.max = logL.max, dev = dev, df.residual = df.residual,
-      nbpar = nbpar, iterations = iterations, code = code, param.ini = param.ini, na.action = na.action)
+      param = param, varparam = varparam, fixed.param = param[seq(along = namb)], random.param = param[-seq(along = namb)],
+      logL = logL, logL.max = logL.max, dev = dev, df.residual = df.residual,
+      nbpar = nbpar, iterations = iterations, code = code, msg = msg, singular.hessian = as.numeric(H.singular), param.ini = param.ini,
+      na.action = na.action)
   }
-  
